@@ -1,3 +1,7 @@
+# /// script
+# dependencies = ["pyautogui", "Pillow", "fastmcp"]
+# ///
+
 """
 FastMCP Screenshot Example
 
@@ -6,13 +10,15 @@ Give Claude a tool to capture and view screenshots.
 
 import io
 
-from fastmcp import FastMCP, Image
+from fastmcp import FastMCP
+from fastmcp.utilities.types import Image
 
 # Create server
-mcp = FastMCP("Screenshot Demo", dependencies=["pyautogui", "Pillow"])
+# Dependencies are configured in screenshot.fastmcp.json
+mcp = FastMCP("Screenshot Demo")
 
 
-@mcp.tool()
+@mcp.tool
 def take_screenshot() -> Image:
     """
     Take a screenshot of the user's screen and return it as an image. Use
