@@ -1,38 +1,37 @@
-"""OpenAPI server implementation for FastMCP - refactored for better maintainability."""
+"""Deprecated: Import from fastmcp.server.openapi instead."""
 
-# Import from server
-from .server import FastMCPOpenAPI
+import warnings
 
-# Import from routing
-from .routing import (
-    MCPType,
-    RouteMap,
-    RouteMapFn,
+from fastmcp.server.openapi import (
     ComponentFn,
     DEFAULT_ROUTE_MAPPINGS,
+    FastMCPOpenAPI,
+    MCPType,
+    OpenAPIResource,
+    OpenAPIResourceTemplate,
+    OpenAPITool,
+    RouteMap,
+    RouteMapFn,
     _determine_route_type,
 )
 
-# Import from components
-from .components import (
-    OpenAPITool,
-    OpenAPIResource,
-    OpenAPIResourceTemplate,
+# Deprecated in 2.14 when OpenAPI support was promoted out of experimental
+warnings.warn(
+    "Importing from fastmcp.experimental.server.openapi is deprecated. "
+    "Import from fastmcp.server.openapi instead.",
+    DeprecationWarning,
+    stacklevel=2,
 )
 
-# Export public symbols - maintaining backward compatibility
 __all__ = [
-    # Server
-    "FastMCPOpenAPI",
-    # Routing
-    "MCPType",
-    "RouteMap",
-    "RouteMapFn",
-    "ComponentFn",
     "DEFAULT_ROUTE_MAPPINGS",
-    "_determine_route_type",
-    # Components
-    "OpenAPITool",
+    "ComponentFn",
+    "FastMCPOpenAPI",
+    "MCPType",
     "OpenAPIResource",
     "OpenAPIResourceTemplate",
+    "OpenAPITool",
+    "RouteMap",
+    "RouteMapFn",
+    "_determine_route_type",
 ]
