@@ -5,26 +5,21 @@ from .auth import (
     AccessToken,
     AuthProvider,
 )
+from .providers.debug import DebugTokenVerifier
 from .providers.jwt import JWTVerifier, StaticTokenVerifier
 from .oauth_proxy import OAuthProxy
+from .oidc_proxy import OIDCProxy
 
 
 __all__ = [
-    "AuthProvider",
-    "OAuthProvider",
-    "TokenVerifier",
-    "JWTVerifier",
-    "StaticTokenVerifier",
-    "RemoteAuthProvider",
     "AccessToken",
+    "AuthProvider",
+    "DebugTokenVerifier",
+    "JWTVerifier",
+    "OAuthProvider",
     "OAuthProxy",
+    "OIDCProxy",
+    "RemoteAuthProvider",
+    "StaticTokenVerifier",
+    "TokenVerifier",
 ]
-
-
-def __getattr__(name: str):
-    # Defer import because it raises a deprecation warning
-    if name == "BearerAuthProvider":
-        from .providers.bearer import BearerAuthProvider
-
-        return BearerAuthProvider
-    raise AttributeError(f"module '{__name__}' has no attribute '{name}'")
