@@ -1,88 +1,153 @@
 from __future__ import annotations as _annotations
 
 import inspect
+import os
 import warnings
+from datetime import timedelta
 from pathlib import Path
 from typing import TYPE_CHECKING, Annotated, Any, Literal
 
+from platformdirs import user_data_dir
 from pydantic import Field, ImportString, field_validator
-from pydantic.fields import FieldInfo
 from pydantic_settings import (
     BaseSettings,
-    EnvSettingsSource,
-    PydanticBaseSettingsSource,
     SettingsConfigDict,
 )
-from typing_extensions import Self
 
 from fastmcp.utilities.logging import get_logger
 
 logger = get_logger(__name__)
 
+ENV_FILE = os.getenv("FASTMCP_ENV_FILE", ".env")
+
 LOG_LEVEL = Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
 
 DuplicateBehavior = Literal["warn", "error", "replace", "ignore"]
 
+TEN_MB_IN_BYTES = 1024 * 1024 * 10
+
 if TYPE_CHECKING:
     from fastmcp.server.auth.auth import AuthProvider
 
 
-class ExtendedEnvSettingsSource(EnvSettingsSource):
-    """
-    A special EnvSettingsSource that allows for multiple env var prefixes to be used.
+class DocketSettings(BaseSettings):
+    """Docket worker configuration."""
 
-    Raises a deprecation warning if the old `FASTMCP_SERVER_` prefix is used.
-    """
+    model_config = SettingsConfigDict(
+        env_prefix="FASTMCP_DOCKET_",
+        extra="ignore",
+    )
 
-    def get_field_value(
-        self, field: FieldInfo, field_name: str
-    ) -> tuple[Any, str, bool]:
-        if prefixes := self.config.get("env_prefixes"):
-            for prefix in prefixes:
-                self.env_prefix = prefix
-                env_val, field_key, value_is_complex = super().get_field_value(
-                    field, field_name
-                )
-                if env_val is not None:
-                    if prefix == "FASTMCP_SERVER_":
-                        # Deprecated in 2.8.0
-                        logger.warning(
-                            "Using `FASTMCP_SERVER_` environment variables is deprecated. Use `FASTMCP_` instead.",
-                        )
-                    return env_val, field_key, value_is_complex
+    name: Annotated[
+        str,
+        Field(
+            description=inspect.cleandoc(
+                """
+                Name for the Docket queue. All servers/workers sharing the same name
+                and backend URL will share a task queue.
+                """
+            ),
+        ),
+    ] = "fastmcp"
 
-        return super().get_field_value(field, field_name)
+    url: Annotated[
+        str,
+        Field(
+            description=inspect.cleandoc(
+                """
+                URL for the Docket backend. Supports:
+                - memory:// - In-memory backend (single process only)
+                - redis://host:port/db - Redis/Valkey backend (distributed, multi-process)
 
+                Example: redis://localhost:6379/0
 
-class ExtendedSettingsConfigDict(SettingsConfigDict, total=False):
-    env_prefixes: list[str] | None
+                Default is memory:// for single-process scenarios. Use Redis or Valkey
+                when coordinating tasks across multiple processes (e.g., additional
+                workers via the fastmcp tasks CLI).
+                """
+            ),
+        ),
+    ] = "memory://"
+
+    worker_name: Annotated[
+        str | None,
+        Field(
+            description=inspect.cleandoc(
+                """
+                Name for the Docket worker. If None, Docket will auto-generate
+                a unique worker name.
+                """
+            ),
+        ),
+    ] = None
+
+    concurrency: Annotated[
+        int,
+        Field(
+            description=inspect.cleandoc(
+                """
+                Maximum number of tasks the worker can process concurrently.
+                """
+            ),
+        ),
+    ] = 10
+
+    redelivery_timeout: Annotated[
+        timedelta,
+        Field(
+            description=inspect.cleandoc(
+                """
+                Task redelivery timeout. If a worker doesn't complete
+                a task within this time, the task will be redelivered to another
+                worker.
+                """
+            ),
+        ),
+    ] = timedelta(seconds=300)
+
+    reconnection_delay: Annotated[
+        timedelta,
+        Field(
+            description=inspect.cleandoc(
+                """
+                Delay between reconnection attempts when the worker
+                loses connection to the Docket backend.
+                """
+            ),
+        ),
+    ] = timedelta(seconds=5)
 
 
 class ExperimentalSettings(BaseSettings):
     model_config = SettingsConfigDict(
         env_prefix="FASTMCP_EXPERIMENTAL_",
         extra="ignore",
+        validate_assignment=True,
     )
 
-    enable_new_openapi_parser: Annotated[
-        bool,
-        Field(
-            description=inspect.cleandoc(
-                """
-                Whether to use the new OpenAPI parser. This parser was introduced
-                for testing in 2.11 and will become the default soon.
-                """
-            ),
-        ),
-    ] = False
+    # Deprecated in 2.14 - the new OpenAPI parser is now the default and only parser
+    enable_new_openapi_parser: bool = False
+
+    @field_validator("enable_new_openapi_parser", mode="after")
+    @classmethod
+    def _warn_openapi_parser_deprecated(cls, v: bool) -> bool:
+        if v:
+            warnings.warn(
+                "enable_new_openapi_parser is deprecated. "
+                "The new OpenAPI parser is now the default (and only) parser. "
+                "You can remove this setting.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+        return v
 
 
 class Settings(BaseSettings):
     """FastMCP settings."""
 
-    model_config = ExtendedSettingsConfigDict(
-        env_prefixes=["FASTMCP_", "FASTMCP_SERVER_"],
-        env_file=".env",
+    model_config = SettingsConfigDict(
+        env_prefix="FASTMCP_",
+        env_file=ENV_FILE,
         extra="ignore",
         env_nested_delimiter="__",
         nested_model_default_partial_update=True,
@@ -115,37 +180,7 @@ class Settings(BaseSettings):
             settings = getattr(settings, parent_attr)
         setattr(settings, attr, value)
 
-    @classmethod
-    def settings_customise_sources(
-        cls,
-        settings_cls: type[BaseSettings],
-        init_settings: PydanticBaseSettingsSource,
-        env_settings: PydanticBaseSettingsSource,
-        dotenv_settings: PydanticBaseSettingsSource,
-        file_secret_settings: PydanticBaseSettingsSource,
-    ) -> tuple[PydanticBaseSettingsSource, ...]:
-        # can remove this classmethod after deprecated FASTMCP_SERVER_ prefix is
-        # removed
-        return (
-            init_settings,
-            ExtendedEnvSettingsSource(settings_cls),
-            dotenv_settings,
-            file_secret_settings,
-        )
-
-    @property
-    def settings(self) -> Self:
-        """
-        This property is for backwards compatibility with FastMCP < 2.8.0,
-        which accessed fastmcp.settings.settings
-        """
-        # Deprecated in 2.8.0
-        logger.warning(
-            "Using fastmcp.settings.settings is deprecated. Use fastmcp.settings instead.",
-        )
-        return self
-
-    home: Path = Path.home() / ".fastmcp"
+    home: Path = Path(user_data_dir("fastmcp", appauthor=False))
 
     test_mode: bool = False
 
@@ -161,6 +196,8 @@ class Settings(BaseSettings):
 
     experimental: ExperimentalSettings = ExperimentalSettings()
 
+    docket: DocketSettings = DocketSettings()
+
     enable_rich_tracebacks: Annotated[
         bool,
         Field(
@@ -189,7 +226,6 @@ class Settings(BaseSettings):
     client_raise_first_exceptiongroup_error: Annotated[
         bool,
         Field(
-            default=True,
             description=inspect.cleandoc(
                 """
                 Many MCP components operate in anyio taskgroups, and raise
@@ -202,20 +238,6 @@ class Settings(BaseSettings):
         ),
     ] = True
 
-    resource_prefix_format: Annotated[
-        Literal["protocol", "path"],
-        Field(
-            default="path",
-            description=inspect.cleandoc(
-                """
-                When perfixing a resource URI, either use path formatting (resource://prefix/path)
-                or protocol formatting (prefix+resource://path). Protocol formatting was the default in FastMCP < 2.4;
-                path formatting is current default.
-                """
-            ),
-        ),
-    ] = "path"
-
     client_init_timeout: Annotated[
         float | None,
         Field(
@@ -235,7 +257,6 @@ class Settings(BaseSettings):
     mask_error_details: Annotated[
         bool,
         Field(
-            default=False,
             description=inspect.cleandoc(
                 """
                 If True, error details from user-supplied functions (tool, resource, prompt)
@@ -248,6 +269,22 @@ class Settings(BaseSettings):
         ),
     ] = False
 
+    strict_input_validation: Annotated[
+        bool,
+        Field(
+            description=inspect.cleandoc(
+                """
+                If True, tool inputs are strictly validated against the input
+                JSON schema. For example, providing the string \"10\" to an
+                integer field will raise an error. If False, compatible inputs
+                will be coerced to match the schema, which can increase
+                compatibility. For example, providing the string \"10\" to an
+                integer field will be coerced to 10. Defaults to False.
+                """
+            ),
+        ),
+    ] = False
+
     server_dependencies: list[str] = Field(
         default_factory=list,
         description="List of dependencies to install in the server environment",
@@ -293,7 +330,6 @@ class Settings(BaseSettings):
     include_tags: Annotated[
         set[str] | None,
         Field(
-            default=None,
             description=inspect.cleandoc(
                 """
                 If provided, only components that match these tags will be
@@ -306,7 +342,6 @@ class Settings(BaseSettings):
     exclude_tags: Annotated[
         set[str] | None,
         Field(
-            default=None,
             description=inspect.cleandoc(
                 """
                 If provided, components that match these tags will be excluded
@@ -320,7 +355,6 @@ class Settings(BaseSettings):
     include_fastmcp_meta: Annotated[
         bool,
         Field(
-            default=True,
             description=inspect.cleandoc(
                 """
                 Whether to include FastMCP meta in the server's MCP responses.
@@ -335,7 +369,6 @@ class Settings(BaseSettings):
     mounted_components_raise_on_load_error: Annotated[
         bool,
         Field(
-            default=False,
             description=inspect.cleandoc(
                 """
                 If True, errors encountered when loading mounted components (tools, resources, prompts)
@@ -349,7 +382,6 @@ class Settings(BaseSettings):
     show_cli_banner: Annotated[
         bool,
         Field(
-            default=True,
             description=inspect.cleandoc(
                 """
                 If True, the server banner will be displayed when running the server via CLI.
@@ -360,6 +392,21 @@ class Settings(BaseSettings):
         ),
     ] = True
 
+    check_for_updates: Annotated[
+        Literal["stable", "prerelease", "off"],
+        Field(
+            description=inspect.cleandoc(
+                """
+                Controls update checking when displaying the CLI banner.
+                - "stable": Check for stable releases only (default)
+                - "prerelease": Also check for pre-release versions (alpha, beta, rc)
+                - "off": Disable update checking entirely
+                Set via FASTMCP_CHECK_FOR_UPDATES environment variable.
+                """
+            ),
+        ),
+    ] = "stable"
+
     @property
     def server_auth_class(self) -> AuthProvider | None:
         from fastmcp.utilities.types import get_cached_typeadapter
@@ -378,23 +425,3 @@ class Settings(BaseSettings):
         auth_class = type_adapter.validate_python(self.server_auth)
 
         return auth_class
-
-
-def __getattr__(name: str):
-    """
-    Used to deprecate the module-level Image class; can be removed once it is no longer imported to root.
-    """
-    if name == "settings":
-        import fastmcp
-
-        settings = fastmcp.settings
-        # Deprecated in 2.10.2
-        if settings.deprecation_warnings:
-            warnings.warn(
-                "`from fastmcp.settings import settings` is deprecated. use `fastmcp.settings` instead.",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-        return settings
-
-    raise AttributeError(f"module '{__name__}' has no attribute '{name}'")
