@@ -1,8 +1,9 @@
 from __future__ import annotations
 
 from collections.abc import Sequence
-from typing import Annotated, Any, TypedDict
+from typing import Annotated, Any, TypedDict, cast
 
+from mcp.types import Icon
 from pydantic import BeforeValidator, Field, PrivateAttr
 from typing_extensions import Self, TypeVar
 
@@ -39,6 +40,10 @@ class FastMCPComponent(FastMCPBaseModel):
         default=None,
         description="The description of the component.",
     )
+    icons: list[Icon] | None = Field(
+        default=None,
+        description="Optional list of icons for this component to display in user interfaces.",
+    )
     tags: Annotated[set[str], BeforeValidator(_convert_set_default_none)] = Field(
         default_factory=set,
         description="Tags for the component.",
@@ -91,7 +96,7 @@ class FastMCPComponent(FastMCPBaseModel):
 
         return meta or None
 
-    def model_copy(
+    def model_copy(  # type: ignore[override]
         self,
         *,
         update: dict[str, Any] | None = None,
@@ -112,7 +117,7 @@ class FastMCPComponent(FastMCPBaseModel):
         copy = super().model_copy(update=update, deep=deep)
         if key is not None:
             copy._key = key
-        return copy
+        return cast(Self, copy)
 
     def __eq__(self, other: object) -> bool:
         if type(self) is not type(other):
@@ -132,7 +137,7 @@ class FastMCPComponent(FastMCPBaseModel):
         """Disable the component."""
         self.enabled = False
 
-    def copy(self) -> Self:
+    def copy(self) -> Self:  # type: ignore[override]
         """Create a copy of the component."""
         return self.model_copy()
 
@@ -168,7 +173,7 @@ class MirroredComponent(FastMCPComponent):
             )
         super().disable()
 
-    def copy(self) -> Self:
+    def copy(self) -> Self:  # type: ignore[override]
         """Create a copy of the component that can be modified."""
         # Create a copy and mark it as not mirrored
         copied = self.model_copy()
