@@ -1,17 +1,20 @@
 import asyncio
+import contextlib
 import sys
-from typing import cast
-from unittest.mock import AsyncMock
+from collections.abc import AsyncIterator
+from typing import Any, cast
 
-import mcp
+import anyio
 import pytest
-from mcp import McpError
+from mcp import ClientSession, McpError
 from mcp.client.auth import OAuthClientProvider
 from pydantic import AnyUrl
 
+import fastmcp
 from fastmcp.client import Client
 from fastmcp.client.auth.bearer import BearerAuth
 from fastmcp.client.transports import (
+    ClientTransport,
     FastMCPTransport,
     MCPConfigTransport,
     SSETransport,
@@ -147,6 +150,47 @@ async def test_call_tool_mcp(fastmcp_server):
         assert "Hello, World!" in content_str
 
 
+async def test_call_tool_with_meta():
+    """Test that meta parameter is properly passed from client to server."""
+    server = FastMCP("MetaTestServer")
+
+    # Create a tool that accesses the meta from the request context
+    @server.tool
+    def check_meta() -> dict[str, Any]:
+        """A tool that returns the meta from the request context."""
+        from fastmcp.server.dependencies import get_context
+
+        context = get_context()
+        assert context.request_context is not None
+        meta = context.request_context.meta
+
+        # Return the meta data as a dict
+        if meta is not None:
+            return {
+                "has_meta": True,
+                "user_id": getattr(meta, "user_id", None),
+                "trace_id": getattr(meta, "trace_id", None),
+            }
+        return {"has_meta": False}
+
+    client = Client(transport=FastMCPTransport(server))
+
+    async with client:
+        # Test with meta parameter - verify the server receives it
+        test_meta = {"user_id": "test-123", "trace_id": "abc-def"}
+        result = await client.call_tool("check_meta", {}, meta=test_meta)
+
+        assert result.data["has_meta"] is True
+        assert result.data["user_id"] == "test-123"
+        assert result.data["trace_id"] == "abc-def"
+
+        # Test without meta parameter - verify fields are not present
+        result_no_meta = await client.call_tool("check_meta", {})
+        # When meta is not provided, custom fields should not be present
+        assert result_no_meta.data.get("user_id") is None
+        assert result_no_meta.data.get("trace_id") is None
+
+
 async def test_list_resources(fastmcp_server):
     """Test listing resources with InMemoryClient."""
     client = Client(transport=FastMCPTransport(fastmcp_server))
@@ -373,21 +417,21 @@ async def test_client_connection(fastmcp_server):
     assert not client.is_connected()
 
 
-async def test_initialize_called_once(fastmcp_server, monkeypatch):
-    mock_initialize = AsyncMock()
-    monkeypatch.setattr(mcp.ClientSession, "initialize", mock_initialize)
+async def test_initialize_called_once(fastmcp_server):
+    """Test that initialization is called once and sets initialize_result."""
     client = Client(transport=FastMCPTransport(fastmcp_server))
     async with client:
-        assert mock_initialize.call_count == 1
+        # Verify that initialization succeeded by checking initialize_result
+        assert client.initialize_result is not None
+        assert client.initialize_result.serverInfo is not None
 
 
 async def test_initialize_result_connected(fastmcp_server):
     """Test that initialize_result returns the correct result when connected."""
     client = Client(transport=FastMCPTransport(fastmcp_server))
 
-    # Initialize result should not be accessible before connection
-    with pytest.raises(RuntimeError, match="Client is not connected"):
-        _ = client.initialize_result
+    # Initialize result should be None before connection
+    assert client.initialize_result is None
 
     async with client:
         # Once connected, initialize_result should be available
@@ -400,21 +444,19 @@ async def test_initialize_result_connected(fastmcp_server):
 
 
 async def test_initialize_result_disconnected(fastmcp_server):
-    """Test that initialize_result raises an error when not connected."""
+    """Test that initialize_result is None when not connected."""
     client = Client(transport=FastMCPTransport(fastmcp_server))
 
-    # Initialize result should not be accessible before connection
-    with pytest.raises(RuntimeError, match="Client is not connected"):
-        _ = client.initialize_result
+    # Initialize result should be None before connection
+    assert client.initialize_result is None
 
     # Connect and then disconnect
     async with client:
         assert client.is_connected()
 
-    # After disconnection, initialize_result should raise an error
+    # After disconnection, initialize_result should be None again
     assert not client.is_connected()
-    with pytest.raises(RuntimeError, match="Client is not connected"):
-        _ = client.initialize_result
+    assert client.initialize_result is None
 
 
 async def test_server_info_custom_version():
@@ -425,6 +467,7 @@ async def test_server_info_custom_version():
 
     async with client:
         result = client.initialize_result
+        assert result is not None
         assert result.serverInfo.name == "CustomVersionServer"
         assert result.serverInfo.version == "1.2.3"
 
@@ -434,12 +477,34 @@ async def test_server_info_custom_version():
 
     async with client:
         result = client.initialize_result
+        assert result is not None
         assert result.serverInfo.name == "DefaultVersionServer"
-        # Should fall back to MCP library version
-        assert result.serverInfo.version is not None
-        assert (
-            result.serverInfo.version != "1.2.3"
-        )  # Should be different from custom version
+        # Should fall back to FastMCP version
+        assert result.serverInfo.version == fastmcp.__version__
+
+
+class _DelayedConnectTransport(ClientTransport):
+    def __init__(
+        self,
+        inner: ClientTransport,
+        connect_started: anyio.Event,
+        allow_connect: anyio.Event,
+    ) -> None:
+        self._inner = inner
+        self._connect_started = connect_started
+        self._allow_connect = allow_connect
+
+    @contextlib.asynccontextmanager
+    async def connect_session(
+        self, **session_kwargs: Any
+    ) -> AsyncIterator[ClientSession]:
+        self._connect_started.set()
+        await self._allow_connect.wait()
+        async with self._inner.connect_session(**session_kwargs) as session:
+            yield session
+
+    async def close(self) -> None:
+        await self._inner.close()
 
 
 async def test_client_nested_context_manager(fastmcp_server):
@@ -472,6 +537,89 @@ async def test_client_nested_context_manager(fastmcp_server):
     assert client._session_state.session is None
 
 
+async def test_client_context_entry_cancelled_starter_cleans_up(fastmcp_server):
+    connect_started = anyio.Event()
+    allow_connect = anyio.Event()
+
+    client = Client(
+        transport=_DelayedConnectTransport(
+            FastMCPTransport(fastmcp_server),
+            connect_started=connect_started,
+            allow_connect=allow_connect,
+        )
+    )
+
+    async def enter_and_never_reach_body() -> None:
+        async with client:
+            pytest.fail(
+                "Context body should not be reached when __aenter__ is cancelled"
+            )
+
+    task = asyncio.create_task(enter_and_never_reach_body())
+    await connect_started.wait()
+
+    task.cancel()
+    with pytest.raises(asyncio.CancelledError):
+        await task
+
+    # Connection startup was cancelled; session state should be fully reset.
+    assert client._session_state.session_task is None
+    assert client._session_state.session is None
+    assert client._session_state.nesting_counter == 0
+
+    # A future connection attempt should work normally.
+    allow_connect.set()
+    async with client:
+        tools = await client.list_tools()
+        assert len(tools) == 3
+
+
+async def test_cancelled_context_entry_waiter_does_not_close_active_session(
+    fastmcp_server,
+):
+    connect_started = anyio.Event()
+    allow_connect = anyio.Event()
+
+    client = Client(
+        transport=_DelayedConnectTransport(
+            FastMCPTransport(fastmcp_server),
+            connect_started=connect_started,
+            allow_connect=allow_connect,
+        )
+    )
+
+    b_done = asyncio.Event()
+    b_started = asyncio.Event()
+
+    async def task_a() -> int:
+        async with client:
+            await b_done.wait()
+            tools = await client.list_tools()
+            return len(tools)
+
+    async def task_b() -> None:
+        b_started.set()
+        async with client:
+            pytest.fail("This context should never be entered due to cancellation")
+
+    a = asyncio.create_task(task_a())
+    await connect_started.wait()
+
+    b = asyncio.create_task(task_b())
+    await b_started.wait()
+    await asyncio.sleep(0)  # let task_b attempt to acquire the client lock
+
+    b.cancel()
+    allow_connect.set()
+
+    with pytest.raises(asyncio.CancelledError):
+        await b
+
+    # task_b is fully cancelled; allow task_a to exercise the connected session.
+    b_done.set()
+    assert await a == 3
+
+
 async def test_concurrent_client_context_managers():
     """
     Test that concurrent client usage doesn't cause cross-task cancel scope issues.
@@ -680,7 +828,8 @@ class TestErrorHandling:
         async with Client(transport=FastMCPTransport(mcp)) as client:
             result = await client.call_tool_mcp("validated_tool", {"x": "abc"})
             assert result.isError
-            assert "'abc' is not of type 'integer'" in result.content[0].text  # type: ignore[attr-defined]
+            # Pydantic validation error message should NOT be masked
+            assert "Input should be a valid integer" in result.content[0].text  # type: ignore[attr-defined]
 
     async def test_specific_tool_errors_are_sent_to_client(self):
         mcp = FastMCP("TestServer")
@@ -943,12 +1092,7 @@ class TestInferTransport:
         transport = infer_transport(config)
         assert isinstance(transport, MCPConfigTransport)
         assert isinstance(transport.transport, FastMCPTransport)
-        assert (
-            len(
-                cast(FastMCP, transport.transport.server)._tool_manager._mounted_servers
-            )
-            == 2
-        )
+        assert len(cast(FastMCP, transport.transport.server)._mounted_servers) == 2
 
     def test_infer_fastmcp_server(self, fastmcp_server):
         """FastMCP server instances should infer to FastMCPTransport."""
@@ -1032,3 +1176,114 @@ class TestAuth:
         assert isinstance(client.transport, SSETransport)
         assert isinstance(client.transport.auth, BearerAuth)
         assert client.transport.auth.token.get_secret_value() == "test_token"
+
+
+class TestInitialize:
+    """Tests for client initialization behavior."""
+
+    async def test_auto_initialize_default(self, fastmcp_server):
+        """Test that auto_initialize=True is the default and works automatically."""
+        client = Client(fastmcp_server)
+
+        async with client:
+            # Should be automatically initialized
+            assert client.initialize_result is not None
+            assert client.initialize_result.serverInfo.name == "TestServer"
+            assert client.initialize_result.instructions is None
+
+    async def test_auto_initialize_explicit_true(self, fastmcp_server):
+        """Test explicit auto_initialize=True."""
+        client = Client(fastmcp_server, auto_initialize=True)
+
+        async with client:
+            assert client.initialize_result is not None
+            assert client.initialize_result.serverInfo.name == "TestServer"
+
+    async def test_auto_initialize_false(self, fastmcp_server):
+        """Test that auto_initialize=False prevents automatic initialization."""
+        client = Client(fastmcp_server, auto_initialize=False)
+
+        async with client:
+            # Should not be automatically initialized
+            assert client.initialize_result is None
+
+    async def test_manual_initialize(self, fastmcp_server):
+        """Test manual initialization when auto_initialize=False."""
+        client = Client(fastmcp_server, auto_initialize=False)
+
+        async with client:
+            # Manually initialize
+            result = await client.initialize()
+
+            assert result is not None
+            assert result.serverInfo.name == "TestServer"
+            assert client.initialize_result is result
+
+    async def test_initialize_idempotent(self, fastmcp_server):
+        """Test that calling initialize() multiple times returns cached result."""
+        client = Client(fastmcp_server, auto_initialize=False)
+
+        async with client:
+            result1 = await client.initialize()
+            result2 = await client.initialize()
+            result3 = await client.initialize()
+
+            # All should return the same cached result
+            assert result1 is result2
+            assert result2 is result3
+
+    async def test_initialize_with_instructions(self):
+        """Test that server instructions are available via initialize_result."""
+        server = FastMCP("InstructionsServer", instructions="Use the greet tool!")
+
+        @server.tool
+        def greet(name: str) -> str:
+            return f"Hello, {name}!"
+
+        client = Client(server)
+
+        async with client:
+            result = client.initialize_result
+            assert result is not None
+            assert result.instructions == "Use the greet tool!"
+
+    async def test_initialize_timeout_custom(self, fastmcp_server):
+        """Test custom timeout for initialize()."""
+        client = Client(fastmcp_server, auto_initialize=False)
+
+        async with client:
+            # Should succeed with reasonable timeout
+            result = await client.initialize(timeout=5.0)
+            assert result is not None
+
+    async def test_initialize_property_after_auto_init(self, fastmcp_server):
+        """Test accessing initialize_result property after auto-initialization."""
+        client = Client(fastmcp_server, auto_initialize=True)
+
+        async with client:
+            # Access via property
+            result = client.initialize_result
+            assert result is not None
+            assert result.serverInfo.name == "TestServer"
+
+            # Call method - should return cached
+            result2 = await client.initialize()
+            assert result is result2
+
+    async def test_initialize_property_before_connect(self, fastmcp_server):
+        """Test that initialize_result property is None before connection."""
+        client = Client(fastmcp_server)
+
+        # Not yet connected
+        assert client.initialize_result is None
+
+    async def test_manual_initialize_can_call_tools(self, fastmcp_server):
+        """Test that manually initialized client can call tools."""
+        client = Client(fastmcp_server, auto_initialize=False)
+
+        async with client:
+            await client.initialize()
+
+            # Should be able to call tools after manual initialization
+            result = await client.call_tool("greet", {"name": "World"})
+            assert "Hello, World!" in str(result.content)
