@@ -1,32 +1,27 @@
 import asyncio
 import json
 import sys
-from collections.abc import Generator
 
 import pytest
-import uvicorn
 from mcp import McpError
-from starlette.applications import Starlette
-from starlette.routing import Mount
 
 from fastmcp.client import Client
 from fastmcp.client.transports import SSETransport
 from fastmcp.server.dependencies import get_http_request
+from fastmcp.server.http import create_sse_app
 from fastmcp.server.server import FastMCP
-from fastmcp.utilities.tests import run_server_in_process
+from fastmcp.utilities.tests import run_server_async
 
 
-def fastmcp_server():
-    """Fixture that creates a FastMCP server with tools, resources, and prompts."""
+def create_test_server() -> FastMCP:
+    """Create a FastMCP server with tools, resources, and prompts."""
     server = FastMCP("TestServer")
 
-    # Add a tool
     @server.tool
     def greet(name: str) -> str:
         """Greet someone by name."""
         return f"Hello, {name}!"
 
-    # Add a second tool
     @server.tool
     def add(a: int, b: int) -> int:
         """Add two numbers together."""
@@ -38,12 +33,10 @@ def fastmcp_server():
         await asyncio.sleep(seconds)
         return f"Slept for {seconds} seconds"
 
-    # Add a resource
     @server.resource(uri="data://users")
     async def get_users():
         return ["Alice", "Bob", "Charlie"]
 
-    # Add a resource template
     @server.resource(uri="data://user/{user_id}")
     async def get_user(user_id: str):
         return {"id": user_id, "name": f"User {user_id}", "active": True}
@@ -51,10 +44,8 @@ def fastmcp_server():
     @server.resource(uri="request://headers")
     async def get_headers() -> dict[str, str]:
         request = get_http_request()
-
         return dict(request.headers)
 
-    # Add a prompt
     @server.prompt
     def welcome(name: str) -> str:
         """Example greeting prompt."""
@@ -63,14 +54,12 @@ def fastmcp_server():
     return server
 
 
-def run_server(host: str, port: int, **kwargs) -> None:
-    fastmcp_server().run(host=host, port=port, **kwargs)
-
-
-@pytest.fixture(autouse=True)
-def sse_server() -> Generator[str, None, None]:
-    with run_server_in_process(run_server, transport="sse") as url:
-        yield f"{url}/sse"
+@pytest.fixture
+async def sse_server():
+    """Start a test server with SSE transport and return its URL."""
+    server = create_test_server()
+    async with run_server_async(server, transport="sse") as url:
+        yield url
 
 
 async def test_ping(sse_server: str):
@@ -91,33 +80,67 @@ async def test_http_headers(sse_server: str):
         assert json_result["x-demo-header"] == "ABC"
 
 
-def run_nested_server(host: str, port: int) -> None:
-    app = fastmcp_server().sse_app(path="/mcp/sse/", message_path="/mcp/messages")
-    mount = Starlette(routes=[Mount("/nest-inner", app=app)])
-    mount2 = Starlette(routes=[Mount("/nest-outer", app=mount)])
-    server = uvicorn.Server(
-        config=uvicorn.Config(app=mount2, host=host, port=port, log_level="error")
+@pytest.fixture
+async def sse_server_custom_path():
+    """Start a test server with SSE on a custom path."""
+    server = create_test_server()
+    async with run_server_async(server, transport="sse", path="/help") as url:
+        yield url
+
+
+@pytest.fixture
+async def nested_sse_server():
+    """Test nested server mounts with SSE."""
+    import uvicorn
+    from starlette.applications import Starlette
+    from starlette.routing import Mount
+
+    from fastmcp.utilities.http import find_available_port
+
+    server = create_test_server()
+    sse_app = create_sse_app(
+        server=server, message_path="/mcp/messages", sse_path="/mcp/sse/"
     )
-    server.run()
+
+    # Nest the app under multiple mounts to test URL resolution
+    inner = Starlette(routes=[Mount("/nest-inner", app=sse_app)])
+    outer = Starlette(routes=[Mount("/nest-outer", app=inner)])
+
+    # Run uvicorn with the nested ASGI app
+    port = find_available_port()
+
+    config = uvicorn.Config(
+        app=outer,
+        host="127.0.0.1",
+        port=port,
+        log_level="critical",
+        ws="websockets-sansio",
+    )
+
+    uvicorn_server = uvicorn.Server(config)
+    server_task = asyncio.create_task(uvicorn_server.serve())
+    await asyncio.sleep(0.1)
+
+    try:
+        yield f"http://127.0.0.1:{port}/nest-outer/nest-inner/mcp/sse/"
+    finally:
+        # Graceful shutdown - required for uvicorn 0.39+ due to context isolation
+        uvicorn_server.should_exit = True
+        await server_task
 
 
-async def test_run_server_on_path():
-    with run_server_in_process(run_server, transport="sse", path="/help") as url:
-        async with Client(transport=SSETransport(f"{url}/help")) as client:
-            result = await client.ping()
-            assert result is True
+async def test_run_server_on_path(sse_server_custom_path: str):
+    """Test running server on a custom path."""
+    async with Client(transport=SSETransport(sse_server_custom_path)) as client:
+        result = await client.ping()
+        assert result is True
 
 
-async def test_nested_sse_server_resolves_correctly():
-    # tests patch for
-    # https://github.com/modelcontextprotocol/python-sdk/pull/659
-
-    with run_server_in_process(run_nested_server) as url:
-        async with Client(
-            transport=SSETransport(f"{url}/nest-outer/nest-inner/mcp/sse/")
-        ) as client:
-            result = await client.ping()
-            assert result is True
+async def test_nested_sse_server_resolves_correctly(nested_sse_server: str):
+    """Test patch for https://github.com/modelcontextprotocol/python-sdk/pull/659"""
+    async with Client(transport=SSETransport(nested_sse_server)) as client:
+        result = await client.ping()
+        assert result is True
 
 
 @pytest.mark.skipif(
@@ -128,18 +151,18 @@ class TestTimeout:
     async def test_timeout(self, sse_server: str):
         with pytest.raises(
             McpError,
-            match="Timed out while waiting for response to ClientRequest. Waited 0.01 seconds",
+            match="Timed out while waiting for response to ClientRequest. Waited 0.03 seconds",
         ):
             async with Client(
                 transport=SSETransport(sse_server),
-                timeout=0.01,
+                timeout=0.03,
             ) as client:
                 await client.call_tool("sleep", {"seconds": 0.1})
 
     async def test_timeout_tool_call(self, sse_server: str):
         async with Client(transport=SSETransport(sse_server)) as client:
             with pytest.raises(McpError, match="Timed out"):
-                await client.call_tool("sleep", {"seconds": 0.1}, timeout=0.01)
+                await client.call_tool("sleep", {"seconds": 0.1}, timeout=0.03)
 
     async def test_timeout_tool_call_overrides_client_timeout_if_lower(
         self, sse_server: str
@@ -149,7 +172,7 @@ class TestTimeout:
             timeout=2,
         ) as client:
             with pytest.raises(McpError, match="Timed out"):
-                await client.call_tool("sleep", {"seconds": 0.1}, timeout=0.01)
+                await client.call_tool("sleep", {"seconds": 0.1}, timeout=0.03)
 
     async def test_timeout_client_timeout_does_not_override_tool_call_timeout_if_lower(
         self, sse_server: str
@@ -163,4 +186,4 @@ class TestTimeout:
             transport=SSETransport(sse_server),
             timeout=0.1,
         ) as client:
-            await client.call_tool("sleep", {"seconds": 0.01}, timeout=2)
+            await client.call_tool("sleep", {"seconds": 0.03}, timeout=2)
