@@ -1,3 +1,4 @@
+import functools
 import json
 import logging
 import uuid
@@ -7,7 +8,7 @@ import pydantic_core
 import pytest
 from inline_snapshot import snapshot
 from mcp.types import ImageContent, TextContent
-from pydantic import BaseModel
+from pydantic import BaseModel, ValidationError
 
 from fastmcp import Context, FastMCP
 from fastmcp.exceptions import NotFoundError, ToolError
@@ -280,8 +281,8 @@ class TestListTools:
         tool_manager.add_tool_transformation(
             "add", ToolTransformConfig(name="add_transformed")
         )
-        tools = await tool_manager.list_tools()
-        tools_by_name = {tool.name: tool for tool in tools}
+        tools_dict = await tool_manager.get_tools()
+        tools_by_name = {tool.name: tool for tool in tools_dict.values()}
         assert "add_transformed" in tools_by_name
         assert "add" not in tools_by_name
 
@@ -303,8 +304,8 @@ class TestListTools:
                 name="add_transformed", description=None, tags={"enabled_tools"}
             ),
         )
-        tools = await tool_manager.list_tools()
-        tools_by_name = {tool.name: tool for tool in tools}
+        tools_dict = await tool_manager.get_tools()
+        tools_by_name = {tool.name: tool for tool in tools_dict.values()}
         assert "add_transformed" in tools_by_name
         assert "add" not in tools_by_name
         assert tools_by_name["add_transformed"].description is None
@@ -472,7 +473,7 @@ class TestCallTools:
         manager = ToolManager()
         tool = Tool.from_function(add)
         manager.add_tool(tool)
-        with pytest.raises(ToolError):
+        with pytest.raises(ValidationError):
             await manager.call_tool("add", {"a": 1})
 
     async def test_call_unknown_tool(self):
@@ -816,6 +817,35 @@ class TestContextHandling:
             ):
                 await manager.call_tool("tool_with_context", {"x": 42})
 
+    async def test_context_with_functools_wraps_decorator(self):
+        """Regression test for #2524: decorated tools with Context should work."""
+
+        def custom_decorator(func):
+            @functools.wraps(func)
+            async def wrapper(*args, **kwargs):
+                return await func(*args, **kwargs)
+
+            return wrapper
+
+        @custom_decorator
+        async def decorated_tool(ctx: Context, query: str) -> str:
+            assert isinstance(ctx, Context)
+            return f"query: {query}"
+
+        manager = ToolManager()
+        tool = Tool.from_function(decorated_tool)
+        manager.add_tool(tool)
+
+        # Verify ctx is excluded from schema
+        assert "ctx" not in json.dumps(tool.parameters)
+
+        mcp = FastMCP()
+        context = Context(fastmcp=mcp)
+
+        async with context:
+            result = await manager.call_tool("decorated_tool", {"query": "test"})
+            assert result.structured_content == {"result": "query: test"}
+
 
 class TestCustomToolNames:
     """Test adding tools with custom names that differ from their function names."""
@@ -1027,12 +1057,12 @@ class TestMountedComponentsRaiseOnLoadError:
 
         # Create a failing mounted server by corrupting it
         parent_mcp.mount(child_mcp, prefix="child")
-        # Corrupt the child server to make it fail during tool loading
-        child_mcp._tool_manager._mounted_servers.append("invalid")  # type: ignore
+        # Corrupt the parent's mounted servers to make it fail during loading
+        parent_mcp._mounted_servers.append("invalid")  # type: ignore
 
-        # Should not raise, just warn
-        tools = await parent_mcp._tool_manager.list_tools()
-        assert isinstance(tools, list)  # Should return empty list, not raise
+        # Should not raise, just warn; use server middleware path now
+        tools = await parent_mcp._list_tools_middleware()
+        assert isinstance(tools, list)  # Should return list, not raise
 
     async def test_mounted_components_raise_on_load_error_true(self):
         """Test that when enabled, mounted component load errors are raised."""
@@ -1041,8 +1071,8 @@ class TestMountedComponentsRaiseOnLoadError:
 
         # Create a failing mounted server
         parent_mcp.mount(child_mcp, prefix="child")
-        # Corrupt the child server to make it fail during tool loading
-        child_mcp._tool_manager._mounted_servers.append("invalid")  # type: ignore
+        # Corrupt the parent's mounted servers to make it fail during loading
+        parent_mcp._mounted_servers.append("invalid")  # type: ignore
 
         # Use temporary settings context manager
         with temporary_settings(mounted_components_raise_on_load_error=True):
@@ -1050,4 +1080,4 @@ class TestMountedComponentsRaiseOnLoadError:
             with pytest.raises(
                 AttributeError, match="'str' object has no attribute 'server'"
             ):
-                await parent_mcp._tool_manager.list_tools()
+                await parent_mcp._list_tools_middleware()
